<?php
/**
 * The icon settings page.
 *
 * @package locrMAPS
 */

declare(strict_types=1);

namespace Locr\WordPress\Plugin\LocrMaps\Admin\SettingsPage;

use Locr\WordPress\Plugin\LocrMaps\Admin\{Database, SettingsPage};
use Locr\WordPress\Plugin\LocrMaps\Admin\SettingsPage\OnlineMaps\{Create, Edit, Main};
use Locr\WordPress\Plugin\LocrMaps\DDD\Repositories\{OnlineMapLocations as OnlineMapLocationsRespository, OnlineMaps as OnlineMapsRepository};
use Locr\WordPress\Plugin\LocrMaps\DDD\ValueObjects\OnlineMap\Id as OnlineMapId;
use Locr\WordPress\Plugin\LocrMaps\DDD\ValueObjects\OnlineMap\Location\Id as LocationId;
use Locr\WordPress\Plugin\LocrMaps\DDD\ValueObjects\SettingsMessageId;
use Locr\WordPress\Plugin\LocrMaps\Plugin;

if ( ! defined( 'ABSPATH' ) ) {
	exit; // Exit if accessed directly.
}

/**
 * This class is responsible for rendering the online maps settings page.
 */
final class OnlineMaps {
	public const NONCE_ACTION_DELETE = 'delete_onlinemap';
	public const SUB_PAGE_NAME       = Plugin::SLUG . '-onlinemaps';

	/**
	 * This function initializes the icon settings page.
	 */
	public static function init(): void {
		self::handle_delete_action();
		self::handle_delete_location_action();
		add_action( 'admin_enqueue_scripts', array( self::class, 'enqueue_scripts' ) );
		add_action( 'admin_notices', array( SettingsPage::class, 'handle_messages' ) );
	}

	/**
	 * This function adds the main menu.
	 */
	public static function add_submenu(): void {
		add_submenu_page(
			parent_slug: SettingsPage::MAIN_PAGE,
			page_title: __( 'ONLINEmaps', 'locrmaps-textdomain' ),
			menu_title: __( 'ONLINEmaps', 'locrmaps-textdomain' ),
			capability: 'manage_options',
			menu_slug: self::SUB_PAGE_NAME,
			callback: array( self::class, 'render_settings_page' )
		);

		do_action( 'locrmaps_after_added_onlinemaps_submenu' );
	}

	/**
	 * This function enqueues the scripts.
	 *
	 * @param string $hook The hook.
	 */
	public static function enqueue_scripts( $hook ): void {
		if ( 'locrmaps_page_' . self::SUB_PAGE_NAME !== $hook ) {
			return;
		}

		wp_enqueue_style(
			handle: 'locrmaps-admin',
			src: plugin_dir_url( dirname( dirname( __DIR__ ) ) ) . 'admin/css/admin.css',
			ver: md5_file( plugin_dir_path( dirname( dirname( __DIR__ ) ) ) . 'admin/css/admin.css' )
		);

		wp_enqueue_script( handle: 'thickbox' );
		wp_enqueue_script( handle: 'wp-api-fetch' );
		wp_enqueue_style( handle: 'thickbox' );
		wp_enqueue_style( handle: 'leaflet-css', src: 'https://unpkg.com/leaflet@' . Plugin::LEAFLET_VERSION . '/dist/leaflet.css', ver: Plugin::LEAFLET_VERSION );
		wp_enqueue_script_module(
			id: self::SUB_PAGE_NAME . '-add-location',
			src: plugin_dir_url( dirname( dirname( __DIR__ ) ) ) . 'admin/js/add_location.js',
			version: md5_file( plugin_dir_path( dirname( dirname( __DIR__ ) ) ) . 'admin/js/add_location.js' ),
		);
	}

	/**
	 * This function handles the delete icon item action.
	 */
	public static function handle_delete_action(): void {
		$page   = (string) ( filter_input( INPUT_GET, 'page', FILTER_DEFAULT ) ?? '' );
		$action = (string) ( filter_input( INPUT_GET, 'action', FILTER_DEFAULT ) ?? '' );

		if ( self::SUB_PAGE_NAME !== $page || 'delete' !== $action ) {
			return;
		}

		$admin_url = admin_url( 'admin.php?page=' . self::SUB_PAGE_NAME );

		$wp_nonce = (string) ( filter_input( INPUT_GET, '_wpnonce', FILTER_DEFAULT ) ?? '' );
		if ( ! wp_verify_nonce( $wp_nonce, self::NONCE_ACTION_DELETE ) ) {
			wp_safe_redirect( add_query_arg( 'message', SettingsMessageId::InvalidNonce->value, $admin_url ) );
			exit;
		}

		$id = intval( filter_input( INPUT_GET, 'id', FILTER_VALIDATE_INT ) ?? 0 );
		if ( ! $id ) {
			return;
		}

		$icons_repository = new OnlineMapsRepository();

		try {
			$icons_repository->delete_by_id( new OnlineMapId( $id ) );
			wp_safe_redirect( add_query_arg( 'message', SettingsMessageId::OnlineMapDeleted->value, $admin_url ) );
			exit;
		} catch ( \Exception $exc ) {
			wp_safe_redirect( add_query_arg( 'message', SettingsMessageId::OnlineMapNotDeleted->value, $admin_url ) );
			exit;
		}
	}

	/**
	 * This function handles the delete icon item action.
	 */
	public static function handle_delete_location_action(): void {
		$page   = (string) ( filter_input( INPUT_GET, 'page', FILTER_DEFAULT ) ?? '' );
		$action = (string) ( filter_input( INPUT_GET, 'action', FILTER_DEFAULT ) ?? '' );

		if ( self::SUB_PAGE_NAME !== $page || 'delete-location' !== $action ) {
			return;
		}

		$admin_url = admin_url( 'admin.php?page=' . self::SUB_PAGE_NAME );

		$referer = (string) ( filter_input( INPUT_SERVER, 'HTTP_REFERER', FILTER_DEFAULT ) ?? '' );
		if ( empty( $referer ) && isset( $_SERVER['HTTP_REFERER'] ) ) {
			$referer = (string) filter_var( wp_unslash( $_SERVER['HTTP_REFERER'] ), FILTER_DEFAULT );
		}
		if ( ! empty( $referer ) ) {
			$admin_url = $referer;
		}

		$wp_nonce = (string) ( filter_input( INPUT_GET, '_wpnonce', FILTER_DEFAULT ) ?? '' );
		if ( ! wp_verify_nonce( $wp_nonce, 'delete_onlinemap_locations_item' ) ) {
			wp_safe_redirect( add_query_arg( 'message', SettingsMessageId::InvalidNonce->value, $admin_url ) );
			exit;
		}

		$id = intval( filter_input( INPUT_GET, 'id', FILTER_VALIDATE_INT ) ?? 0 );
		if ( ! $id ) {
			return;
		}

		$locations_repository = new OnlineMapLocationsRespository();

		try {
			$locations_repository->delete_by_id( new LocationId( $id ) );
			wp_safe_redirect( add_query_arg( 'message', SettingsMessageId::OnlineMapLocationDeleted->value, $admin_url ) );
			exit;
		} catch ( \Exception $exc ) {
			wp_safe_redirect( add_query_arg( 'message', SettingsMessageId::OnlineMapLocationNotDeleted->value, $admin_url ) );
			exit;
		}
	}

	/**
	 * This function renders the settings page for the online maps.
	 */
	public static function render_settings_page(): void {
		Database::init_onlinemaps_table();
		Database::init_onlinemap_locations_table();

		$action = (string) ( filter_input( INPUT_GET, 'action', FILTER_DEFAULT ) ?? '' );
		match ( $action ) {
			'create' => Create::process(),
			'edit' => Edit::render(),
			default => Main::render(),
		};
	}
}
